<?php
/**
 * Blocks
 *
 * @package iconic-woo-quickview
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Class Iconic_WQV_Blocks
 */
class Iconic_WQV_Blocks {

	/**
	 * Blocks to override.
	 *
	 * @var array
	 */
	public static $blocks_to_override = array(
		'woocommerce/product-category',
		'woocommerce/product-tag',
		'woocommerce/products-by-attribute',
		'woocommerce/handpicked-products',
		'woocommerce/product-best-sellers',
		'woocommerce/product-new',
		'woocommerce/product-top-rated',
		'woocommerce/product-on-sale',
	);

	/**
	 * Init
	 */
	public static function run() {
		add_filter( 'render_block', array( __CLASS__, 'modify_shop_loop_block_render' ), 15, 3 );
		add_filter( 'woocommerce_blocks_product_grid_item_html', array( __CLASS__, 'modify_product_grid_item_html' ), 10, 3 );
	}

	/**
	 * Modify shop loop block render.
	 * 
	 * Relates to the Query Loop, Products (Beta) and Collections (Beta) blocks.
	 *
	 * @param string   $block_content The block content.
	 * @param array    $block         The full block, including name and attributes.
	 * @param WP_Block $instance      The block instance.
	 * 
	 * @return string
	 */
	public static function modify_shop_loop_block_render( $block_content, $block, $block_instance ) {
		global $jckqv;

		if ( is_admin() ) {
			return $block_content;
		}

		$auto_insert      = Iconic_WQV_Core_Settings::get_setting( 'trigger_position_autoinsert' );
		$trigger_position = Iconic_WQV_Core_Settings::get_setting( 'trigger_position_position' );

		if ( ! $auto_insert || ! $trigger_position ) {
			return $block_content;
		}

		$setting_to_block_map = array(
			// Block based shop loops don't have the same
			// element as non-block based loops, so we need
			// to output before the featured image.
			'beforeitem'  => array( 'woocommerce/product-image' ),
			'beforetitle' => array( 'core/post-title' ),
			'aftertitle'  => array( 'core/post-title' ),
			'afteritem'   => array( 'woocommerce/product-button' ),
		);

		if ( 
			empty( $block['blockName'] ) || 
			( 
				! empty( $setting_to_block_map[ $trigger_position ] ) && 
				! in_array( $block['blockName'], $setting_to_block_map[ $trigger_position ], true )
			)
		) {
			return $block_content;
		}

		if ( empty( $block_instance->context['postId'] ) ) {
			return $block_content;
		}

		if ( ! empty( $block_instance->context['postType'] ) && 'product' !== $block_instance->context['postType'] ) {
			return $block_content;
		}

		ob_start();
		$jckqv->display_button( $block_instance->context['postId'] );
		$button = ob_get_clean();

		if ( str_contains( $trigger_position, 'before' ) ) {
			$block_content = $button . $block_content;
		} else {
			$block_content = $block_content . $button;
		}

		return $block_content;
	}

	/**
	 * Modify product grid item HTML.
	 * 
	 * Relates to the other WC product blocks, excluding Products (Beta), 
	 * Collections (Beta) blocks as well as the Query Loop block.
	 *
	 * @param string $html Product grid item HTML.
	 * @param array $data Product data passed to the template.
	 * @param \WC_Product $product Product object.
	 * 
	 * @return string Updated product grid item HTML.
	 */
	public static function modify_product_grid_item_html( $html, $data, $product ) {
		global $jckqv;

		if ( is_admin() ) {
			return $html;
		}

		$auto_insert      = Iconic_WQV_Core_Settings::get_setting( 'trigger_position_autoinsert' );
		$trigger_position = Iconic_WQV_Core_Settings::get_setting( 'trigger_position_position' );

		if ( ! $auto_insert || ! $trigger_position ) {
			return $html;
		}

		ob_start();
		$jckqv->display_button( $product->get_id() );
		$button = ob_get_clean();

		switch ( $trigger_position ) {
			case 'beforeitem':
				$pattern = '/<a href=".*?" class="wc-block-grid__product-link">/i';
				break;

			case 'beforetitle':
			case 'aftertitle':
				$pattern = '/<div class="wc-block-grid__product-title">.*?<\/div>/i';
				break;

			case 'afteritem':
				$pattern = '/<div class="wp-block-button wc-block-grid__product-add-to-cart">.*?<\/div>/i';
				break;
		}

		$html = preg_replace_callback(
			$pattern,
			function( $matches ) use ( $button, $trigger_position ) {
				if ( str_contains( $trigger_position, 'before' ) ) {
					return $button . $matches[0];
				} else {
					return $matches[0] . $button;
				}
			},
			$html
		);
		
		return $html;
	}

	/**
	 * Extract the product ID from the WC single product block.
	 *
	 * @param WP_Post $post WP_Post object instance.
	 * 
	 * @return bool|int
	 */
	public static function get_single_product_block_product_id( $post ) {
		$product_id = false;
 
		if ( $post && has_block( 'woocommerce/single-product', $post ) ) {
			
			foreach( parse_blocks( $post->post_content ) as $block ) {
				if ( 
					'woocommerce/single-product' === $block['blockName'] &&
					! empty( $block['attrs']['productId'] )
				) {
					$product_id = absint( $block['attrs']['productId'] );
					break;
				}
			}
		}

		return $product_id;
	}
}
